package com.prorace.viewcontroller;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import android.app.DatePickerDialog;
import android.app.DatePickerDialog.OnDateSetListener;
import android.app.TimePickerDialog;
import android.app.TimePickerDialog.OnTimeSetListener;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TimePicker;
import android.widget.Toast;


public class ActivityCreate extends FragmentActivity{

	//Race attributes
	private String raceName;
	private String raceDiscipline;
	private int raceLaps;
	private String raceDateTime;
	private int raceLimitTime;
	private int raceMaxCompetitors;
	
	//xmlVariables
	EditText textName, textDate, textTime;
	Spinner disciplinesMenu, lapsMenu, maxCompetitorsMenu, limitTimeMenu;
	
	//calendar
	Calendar c;
	
	//shape map button
	Button buttonMap;
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_create);
		
		initialize();

	}
	
	public void initialize(){

		//defining name text box
		textName = (EditText) findViewById(R.id.name);

		//defining disciplines spinner
		disciplinesMenu = (Spinner) findViewById(R.id.disciplines);
		ArrayAdapter <String> disciplinesList = new ArrayAdapter <String> (this, android.R.layout.simple_spinner_item, getResources().getStringArray(R.array.disciplines));
		disciplinesMenu.setAdapter(disciplinesList);
		
		//defining laps spinner
		lapsMenu = (Spinner) findViewById(R.id.laps);
		ArrayAdapter <Integer> lapsList = convertIntVectorToArrayAdapterInteger(getResources().getIntArray(R.array.laps));
		lapsMenu.setAdapter(lapsList);
		
		//defining max competitors spinner
		maxCompetitorsMenu = (Spinner) findViewById(R.id.maxCompetitors);
		ArrayAdapter <Integer> maxCompetitorsList = convertIntVectorToArrayAdapterInteger(getResources().getIntArray(R.array.maxCompetitors));
		maxCompetitorsMenu.setAdapter(maxCompetitorsList);
		
		//get limit time
		limitTimeMenu = (Spinner) findViewById(R.id.limitTime);
		ArrayAdapter <Integer> limitTimeList = convertIntVectorToArrayAdapterInteger(getResources().getIntArray(R.array.limitTime));
		limitTimeMenu.setAdapter(limitTimeList);
		
		//getting time and date from system
		c = Calendar.getInstance();
		
		//get date
		textDate = (EditText) findViewById(R.id.date);
		textDate.setText(c.get(Calendar.YEAR) + "/" + new DecimalFormat("00").format((c.get(Calendar.MONTH)+1)) + "/" +new DecimalFormat("00").format( c.get(Calendar.DAY_OF_MONTH)));

		//get time
		textTime = (EditText) findViewById(R.id.time);
		if(c.get(Calendar.HOUR_OF_DAY) == 23)
			textTime.setText("00:" + new DecimalFormat("00").format(c.get(Calendar.MINUTE)));
		else
			textTime.setText(new DecimalFormat("00").format((c.get(Calendar.HOUR_OF_DAY)+1)) + ":" + new DecimalFormat("00").format(c.get(Calendar.MINUTE)));
		
		//shape map button
		buttonMap = (Button) findViewById (R.id.map);
	}
	
	public ArrayAdapter<Integer> convertIntVectorToArrayAdapterInteger(int[] intVector){
		Integer [] integerVector = new Integer[intVector.length];
		for(int i=0; i<intVector.length; i++)
			integerVector [i] = Integer.valueOf(intVector[i]);
		return new ArrayAdapter <Integer> (this, android.R.layout.simple_spinner_item, integerVector);
		
	}
	
	public void onClick_date (View v){
		new DatePickerDialog(v.getContext(), new mDateStateListener(), c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH)).show();
	}
	
	public void onClick_time (View v){
		new TimePickerDialog(v.getContext(), new mTimeStateListener(), c.get(Calendar.HOUR_OF_DAY), c.get(Calendar.MINUTE), true).show();
	}
	
	public void onClick_map (View v){
		if (getValues())
			putParametersAndGo();
	}
	
	public boolean getValues(){
		raceName = textName.getText().toString();
		if (raceName.isEmpty()){
			textName.setError(getString(R.string.createPleaseInputANameForYourRaceHere));
			return false;
		}
		
		raceDiscipline = disciplinesMenu.getSelectedItem().toString();
		
		raceLaps = Integer.valueOf(lapsMenu.getSelectedItem().toString());
		
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm", java.util.Locale.getDefault());
		try{
			java.util.Date controllerDateTime;
			controllerDateTime = sdf.parse(textDate.getText().toString() + " "+ textTime.getText().toString() + ":00");
			if(controllerDateTime.before(new java.util.Date())){
				Toast.makeText(getApplicationContext(), getString(R.string.createPleaseSelectADateInTheFuture), Toast.LENGTH_LONG).show();
				textDate.setError(getString(R.string.createPleaseSelectADateInTheFuture));
				textTime.setError(getString(R.string.createPleaseSelectADateInTheFuture));
				return false;
			}
		} catch (java.text.ParseException pe) {
			textDate.setError(getString(R.string.createPleaseFormatDateAndTimeCorrectly));
			textTime.setError(getString(R.string.createPleaseFormatDateAndTimeCorrectly));
			return false;
		}
		raceDateTime = textDate.getText().toString() + " "+ textTime.getText().toString() + ":00";
		
		raceLimitTime = Integer.valueOf(limitTimeMenu.getSelectedItem().toString());
		
		raceMaxCompetitors = Integer.valueOf(maxCompetitorsMenu.getSelectedItem().toString());
		
		return true;
	}
	
	public void putParametersAndGo(){
		Intent i = new Intent();
		i.setClass(getApplicationContext(), ActivityCreateMap.class);
		i.putExtra("name", raceName);
		i.putExtra("discipline", raceDiscipline);
		i.putExtra("laps", raceLaps);
		i.putExtra("dateTime", raceDateTime);
		i.putExtra("limitTime", raceLimitTime);
		i.putExtra("maxCompetitors", raceMaxCompetitors);
		startActivity(i);		
	}
	
	public class mDateStateListener implements OnDateSetListener {
		@Override
		public void onDateSet(DatePicker view, int year, int month, int day) {
			textDate.setText(year + "/" + new DecimalFormat("00").format((month+1)) + "/" + new DecimalFormat("00").format(day));
		}
	}
	
	public class mTimeStateListener implements OnTimeSetListener {
		@Override
		public void onTimeSet(TimePicker view, int hour, int minute) {
			textTime.setText(new DecimalFormat("00").format(hour) + ":" + new DecimalFormat("00").format(minute));
		}
	}
}