package com.prorace.viewcontroller;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Vector;

import com.google.android.gms.maps.*;
import com.google.android.gms.maps.GoogleMap.OnMapClickListener;
import com.google.android.gms.maps.GoogleMap.OnMapLongClickListener;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.GoogleMap.OnMarkerDragListener;
import com.google.android.gms.maps.model.*;

import com.prorace.model.CreateRace;
import com.prorace.model.ErrorCode;
import com.prorace.jpa.*;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class ActivityCreateMap extends FragmentActivity implements OnMapReadyCallback, OnMapClickListener, OnMapLongClickListener, OnMarkerClickListener, OnMarkerDragListener {
	
	private final static int START = 1;
	private final static int FINISH = 2;
	private final static int TRAVEL = 3;
	private final static int DEFAULT_BUTTON_PRESSED = START;
	
	//race parameters
	Race race;
	
	//variable for map management
	GoogleMap map;
	TextView selection;
	Marker startMarker, finishMarker;
	java.util.Vector <Marker> travelMarkers;
	
	//buttons
	Button startB, travelB, finishB, beginRace;
	int buttonPressed = DEFAULT_BUTTON_PRESSED;
	
	
	
    protected void onCreate(Bundle savedInstanceState) {
    	
    	super.onCreate(savedInstanceState);
    	setContentView(R.layout.activity_create_map);

    	initialize();
    	
    	try {
			getParameters();
		} catch (ParseException e) {
			AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);
        	alertDialog.setTitle("ERROR");
        	alertDialog.setMessage(ErrorCode.ERROR_PASSING_PARAMETERS_CREATE_RACE.toString());
        	alertDialog.setPositiveButton("OK", new DialogInterface.OnClickListener(){
        		public void onClick(DialogInterface dialog, int i){
        			finish();
        		}
        	});
        	alertDialog.show();
			e.printStackTrace();
		}

    }
    
    public void initialize(){
    	startB = (Button) findViewById(R.id.createmapStart);
    	finishB = (Button) findViewById(R.id.createmapFinish);
    	travelB = (Button) findViewById(R.id.createmapTravel);
    	selection = (TextView) findViewById(R.id.createmapSelectedButton);
    	beginRace = (Button) findViewById(R.id.createmapBeginRace);
    	
    	MapFragment mapFragment = (MapFragment) getFragmentManager().findFragmentById(R.id.mapCreate);
    	mapFragment.getMapAsync(this);
    	
    	travelMarkers = new Vector<Marker>();
    	
    	race = new Race();
    }
        
    public void getParameters() throws ParseException{
    	Bundle extras = getIntent().getExtras();

        race.setName(extras.getString("name"));
        race.setDiscipline(extras.getString("discipline"));
        race.setLaps(extras.getInt("laps"));
        race.setLimitTime(extras.getInt("limitTime"));
        race.setMaxCompetitors(extras.getInt("maxCompetitors"));
        
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm", java.util.Locale.getDefault());
        race.setDateTime(sdf.parse(extras.getString("dateTime")));
    }
    
    public void onMapReady(GoogleMap map){
    	LocationManager locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
    	Criteria criteria = new Criteria();
        String provider = locationManager.getBestProvider(criteria, true);
        Location myLocation = locationManager.getLastKnownLocation(provider);
        
		map.setMyLocationEnabled(true);
        map.getUiSettings().setZoomControlsEnabled(true);
        map.getUiSettings().setCompassEnabled(true);
        map.getUiSettings().setAllGesturesEnabled(true);
        map.getUiSettings().setCompassEnabled(true);
        
        map.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(myLocation.getLatitude(), myLocation.getLongitude()), 17));
        
		map.setMapType(GoogleMap.MAP_TYPE_SATELLITE);
		
		map.setOnMapClickListener(this);
		map.setOnMapLongClickListener(this);
		map.setOnMarkerClickListener(this);
		map.setOnMarkerDragListener(this);
		
		this.map = map;

    }
    
    public void onClick_start(View v){
    	buttonPressed = START;
    	selection.setText(R.string.createmapMarkAStartPoint);
    }
    
    public void onClick_finish(View v){
    	buttonPressed = FINISH;
    	selection.setText(R.string.createmapMarkAFinishPoint);    	
    }
    
    public void onClick_travel(View v){
    	buttonPressed = TRAVEL;
    	selection.setText(R.string.createmapTrackMiddlePoints);   	
    }
    
    public void onClick_beginRace(View v){
    	
    	if(startMarker==null){
			Toast.makeText(getApplicationContext(), R.string.createmapPleaseAddStartPoint, Toast.LENGTH_LONG).show();
			return;
    	}
    	if(finishMarker==null){
			Toast.makeText(getApplicationContext(), R.string.createmapPleaseAddFinishPoint, Toast.LENGTH_LONG).show();
			return;
    	}
    	
    	race.setMap(
    				new Map(	new Point(startMarker.getPosition().latitude, startMarker.getPosition().longitude),
    							new Point(finishMarker.getPosition().latitude, finishMarker.getPosition().longitude),
    							getListPointFromMarker()
    						)
    				);
    	
    	ErrorCode ec = CreateRace.createRace(race);
    	
    	if (ec.getCode() == 0){
    		Intent i = new Intent();
			i.setClass(getApplicationContext(), ActivityDisplayRace.class);
			i.putExtra("race", race);
			startActivity(i);
    	}
    	else
    		Toast.makeText(getApplicationContext(), ec.getError(), Toast.LENGTH_LONG).show();
    	
    	
    }
    
    public java.util.List <Point> getListPointFromMarker(){
    	java.util.List <Point> points = new java.util.ArrayList <Point> ();
    	
    	for (int i=0; i<travelMarkers.size(); i++){
    		LatLng position = travelMarkers.elementAt(i).getPosition();
    		points.add(new Point (position.latitude, position.longitude));
    	}
    	
    	return points;
    }
    
    public void onMapClick(LatLng click){
    	Toast.makeText(getApplicationContext(),R.string.createmapLongClickForSettingAPoint , Toast.LENGTH_SHORT).show();
    }
    
	public void onMapLongClick (LatLng click){
	
		switch (buttonPressed){
		
		case FINISH:
			if(finishMarker!=null)
				finishMarker.remove();
				finishMarker = map.addMarker(new MarkerOptions()
								.title(getString(R.string.createmapFinishPoint))
								.position(click)
								.draggable(true)
								.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE)));
			break;
			
		case TRAVEL:
			travelMarkers.add(map.addMarker(new MarkerOptions()
									.title(getString(R.string.createmapTravelPoint) + ": " + travelMarkers.size())
									.position(click)
									.draggable(true)));
			break;
			
		case START:
			if(startMarker != null)
				startMarker.remove();
			startMarker = map.addMarker(new MarkerOptions()
								.title(getString(R.string.createmapStartPoint))
								.position(click)
								.draggable(true)
								.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW)));
			buttonPressed = TRAVEL;
	    	selection.setText(R.string.createmapTrackMiddlePoints);  
			break;
		
		default:
			Toast.makeText(getApplicationContext(),R.string.createmapSelectOneOption , Toast.LENGTH_SHORT).show();
			break;
			
		}
	}
	
	public boolean onMarkerClick (Marker mark){
		if (startMarker != null)
			if (startMarker.equals(mark))
				startMarker = null;
		if (finishMarker != null)
			if (finishMarker.equals(mark))
				finishMarker = null;
		if (travelMarkers.contains(mark))
			travelMarkers.remove(mark);
		mark.remove();
		return true;
	}
	
	@Override
	public void onMarkerDrag(Marker mark) {
	}

	@Override
	public void onMarkerDragEnd(Marker mark) {
		if (startMarker != null)
			if (startMarker.equals(mark))
				startMarker.setPosition(mark.getPosition());
		if (finishMarker != null)
			if (finishMarker.equals(mark))
				finishMarker.setPosition(mark.getPosition());
		if (travelMarkers.contains(mark)){
			travelMarkers.elementAt(travelMarkers.indexOf(mark)).setPosition(mark.getPosition());
		}
	}

	@Override
	public void onMarkerDragStart(Marker mark) {
	}
	
	public void onBackPressed(){
		map.clear();
		super.onBackPressed();
	}
}
